import { Model, SDK, Response } from './common';

module Event {

    export interface EventInterface {

        id?: number;
        begin_date: Date;
        end_date: Date;
        name: string;
        image: string;
    }

    interface CreateEventResponse extends Response {

        event: EventInterface;
    }

    interface GetEventByIdResponse extends Response {

        event: EventInterface;
    }

    interface GetAllEventResponse extends Response {

        events: EventInterface[];
    }

    interface GetActivitiesResponse extends Response {

        activities: any[];
    }

    interface GetAttendeesResponse extends Response {

        attendees: any[];
    }

    interface GetFormsResponse extends Response {

        forms: any[];
    }

    interface GetPartnersResponse extends Response {

        partners: any[];
    }

    interface GetSponsorsResponse extends Response {

        sponsors: any[];
    }

    interface GetSponsorshipTiersResponse extends Response {

        sponsorshipTiers: any[];
    }

    interface GetTeamsResponse extends Response {

        teams: any[];
    }

    interface GetVolunteersResponse extends Response {

        volunteers: any[];
    }

    export async function create(event: EventInterface): Promise<Instance> {
        try {
            let res = <CreateEventResponse>await SDK.getInstance().POST("event", event);
            return new Instance(res.event);
        } catch (err) {
            return null;
        }
    }

    export async function getById(id: number): Promise<Instance> {
        try {
            let res = <GetEventByIdResponse>await SDK.getInstance().GET("event/" + String(id));
            if (res.success && res.event) {
                return new Instance(res.event);
            } else {
                return null;
            }
        } catch (err) {
            return null;
        }
    }

    export async function getAll(): Promise<Instance[]> {
        try {
            let res = <GetAllEventResponse>await SDK.getInstance().GET("event");
            if (res.success && res.events) {
                let events: Instance[] = [];
                for (let event of res.events) {
                    events.push(new Instance(event));
                }
                return events;
            } else {
                return null;
            }
        } catch (err) {
            return null;
        }
    }

    export class Instance implements EventInterface, Model {

        public id: number;
        public begin_date: Date;
        public end_date: Date;
        public name: string;
        public image: string;

        constructor(event: EventInterface) {

            this.id = event.id;
            this.begin_date = new Date(event.begin_date);
            this.end_date = new Date(event.end_date);
            this.name = event.name;
            this.image = event.image;
        }

        public async save(): Promise<boolean> {
            return new Promise<boolean>(async (resolve, reject) => {
                let result = <Response>await SDK.getInstance().PUT("event/" + String(this.id), {
                    begin_date: this.begin_date,
                    end_date: this.end_date,
                    name: this.name,
                    image: this.image
                });
                if (result.success) {
                    resolve(true);
                } else {
                    reject(result.err);
                }
            });
        }

        public async delete(): Promise<boolean> {
            return new Promise<boolean>(async (resolve, reject) => {
                let result = <Response>await SDK.getInstance().DELETE("event/" + String(this.id), {});
                if (result.success) {
                    resolve(true);
                } else {
                    reject(result.err);
                }
            });
        }

        public async getActivities() {
            try {
                let result = <GetActivitiesResponse>await SDK.getInstance()
                    .GET("event/" + String(this.id) + "/activity");
                if (result.success && result.activities) {
                    return result.activities;
                } else {
                    return null;
                }
            } catch (err) {
                return null;
            }
        }

        public async getAttendees() {
            try {
                let result = <GetAttendeesResponse>await SDK.getInstance()
                    .GET("event/" + String(this.id) + "/attendee");
                if (result.success && result.attendees) {
                    return result.attendees;
                } else {
                    return null;
                }
            } catch (err) {
                return null;
            }
        }

        public async getForms() {
            try {
                let result = <GetFormsResponse>await SDK.getInstance()
                    .GET("event/" + String(this.id) + "/form");
                if (result.success && result.forms) {
                    return result.forms;
                } else {
                    return null;
                }
            } catch (err) {
                return null;
            }
        }

        public async getPartners() {
            try {
                let result = <GetPartnersResponse>await SDK.getInstance()
                    .GET("event/" + String(this.id) + "/partner");
                if (result.success && result.partners) {
                    return result.partners;
                } else {
                    return null;
                }
            } catch (err) {
                return null;
            }
        }

        public async getSponsors() {
            try {
                let result = <GetSponsorsResponse>await SDK.getInstance()
                    .GET("event/" + String(this.id) + "/sponsor");
                if (result.success && result.sponsors) {
                    return result.sponsors;
                } else {
                    return null;
                }
            } catch (err) {
                return null;
            }
        }

        public async getSponsorshipTiers() {
            try {
                let result = <GetSponsorshipTiersResponse>await SDK.getInstance()
                    .GET("event/" + String(this.id) + "/sponsorship/tier");
                if (result.success && result.sponsorshipTiers) {
                    return result.sponsorshipTiers;
                } else {
                    return null;
                }
            } catch (err) {
                return null;
            }
        }

        public async getTeams() {
            try {
                let result = <GetTeamsResponse>await SDK.getInstance()
                    .GET("event/" + String(this.id) + "/team");
                if (result.success && result.teams) {
                    return result.teams;
                } else {
                    return null;
                }
            } catch (err) {
                return null;
            }
        }

        public async getVolunteers() {
            try {
                let result = <GetVolunteersResponse>await SDK.getInstance()
                    .GET("event/" + String(this.id) + "/volunteer");
                if (result.success && result.volunteers) {
                    return result.volunteers;
                } else {
                    return null;
                }
            } catch (err) {
                return null;
            }
        }
    }
}
export = Event;
